declare module '@svelte-put/async-stack' {
	import type { Component, ComponentProps } from 'svelte';
	import type { ActionReturn } from 'svelte/action';
	class StackItem<Resolved, UserComponent extends import("svelte").Component = import("svelte").Component<StackItemProps<Resolved>, any, string>> {
		
		constructor(config: Required<StackItemInstanceConfig<Resolved, string, UserComponent>>);
		
		state: StackItemState;
		
		config: Required<StackItemInstanceConfig<Resolved, string, UserComponent>>;
		/**
		 * a promise that resolves when the item is popped or resolved via the .resolve method
		 * */
		resolution: Promise<Resolved | undefined>;
		resume: () => void;
		pause: () => void;
		
		resolve: (resolved?: Resolved | undefined) => Promise<Resolved | undefined>;
		#private;
	}
	class Stack<VariantMap extends Record<string, import("svelte").Component> = {}> {
		
		constructor(variantConfigMap: Record<keyof VariantMap, StackItemVariantConfig<any, any, any>>, init?: StackItemCommonConfig<any, any, any> | undefined);
		/**
		 * the stack items
		 * */
		items: StackItem<any>[];
		
		config: Required<StackItemCommonConfig<any, any, any>>;
		actions: {
			/**
			 * register the element to render a stack item into
			 * */
			render: (node: HTMLElement, item: StackItem<any>) => StackItemRenderActionReturn;
		};
		
		push<Variant extends Extract<keyof VariantMap, string>, UserComponent extends VariantMap[Variant] = VariantMap[Variant], Resolved = Awaited<import("svelte").ComponentProps<UserComponent>["item"]["resolution"]> | undefined>(variant: Variant, config?: StackItemByVariantPushConfig<Resolved, Variant, UserComponent> | undefined): StackItem<Resolved>;
		
		push<UserComponent extends VariantMap[Variant] = VariantMap[Variant], Resolved = Awaited<import("svelte").ComponentProps<UserComponent>["item"]["resolution"]> | undefined>(variant: "custom", config: StackItemCustomPushConfig<Resolved, UserComponent>): StackItem<Resolved>;
		
		pop(id?: string | undefined, detail?: any): void;
		
		pop(config?: StackItemPopVerboseInput | undefined): void;
		/**
		 * pause a stack item, if it has a timeout
		 * */
		pause(id: string): void;
		/**
		 * resume a stack item, if it has been paused
		 * */
		resume(id: string): void;
		#private;
	}
	function stack(init?: StackItemCommonConfig<any, any, any> | undefined): StackBuilder;

	class StackBuilder<VariantMap extends Record<string, import("svelte").Component> = {}> {
		
		constructor(init?: StackItemCommonConfig<any, any, any> | undefined);
		/**
		 * add config for a stack item variant
		 * */
		addVariant<Resolved, Variant extends string, UserComponent extends import("svelte").Component>(variant: Variant, config: UserComponent | Omit<StackItemVariantConfig<Resolved, Variant, UserComponent>, "variant">): StackBuilder<VariantMap & Record<Variant, UserComponent>>;
		/**
		 * Build the actual stack
		 * */
		build(): Stack<VariantMap>;
		#private;
	}
	type StackItemCommonConfig<
		Resolved,
		Variant extends string,
		UserComponent extends Component,
	> = {
		/**
		 * milliseconds to wait and automatically pop the stack item.
		 * Defaults to `0` (disabled)
		 */
		timeout?: number;
		/**
		 * id generator for stack items. Defaults to 'uuid'
		 *
		 *   - counter: use an auto-incremented counter that is scoped to the
		 *   - uuid: use `crypto.randomUUID()`, fallback to `counter` if not available
		 *   - callback: a custom function that accepts {@link StackItemInstanceConfig} and returns a string as the id
		 */
		id?:
			| 'counter'
			| 'uuid'
			| ((
					config: Required<Omit<StackItemInstanceConfig<Resolved, Variant, UserComponent>, 'id'>>,
			  ) => string);
	};

	type StackItemProps<Resolved> = {
		/** the stack item instance injected by the stack */
		item: Omit<StackItem<Resolved>, '#internals'>;
	};

	/** predefined variant config provided while building a {@link Stack} instance */
	type StackItemVariantConfig<
		Resolved,
		Variant extends string,
		UserComponent extends Component,
	> = StackItemCommonConfig<Resolved, Variant, UserComponent> & {
		/** string variant representing this config, must be unique within a {@link Stack} instance  */
		variant: Variant;
		/** any Svelte component used for rendering stack item UI */
		component: UserComponent;
		/** inferred props from `component` */
		props?: Omit<ComponentProps<UserComponent>, 'item'>;
	};

	/** a resolved config for a {@link StackItemInstance} */
	type StackItemInstanceConfig<
		Resolved,
		Variant extends string,
		UserComponent extends Component,
	> = Required<Omit<StackItemVariantConfig<Resolved, Variant, UserComponent>, 'id'>> & {
		id: string;
		timeout: number;
	};

	type StackItemState = 'idle' | 'elapsing' | 'paused' | 'timeout' | 'resolved';

	type StackItemByVariantPushConfig<
		Resolved,
		Variant extends string,
		UserComponent extends Component,
	> = StackItemCommonConfig<Resolved, Variant, UserComponent> & {
		props?: Omit<ComponentProps<UserComponent>, 'item'>;
	};

	type StackItemCustomPushConfig<
		Resolved,
		UserComponent extends Component,
	> = StackItemCommonConfig<Resolved, 'custom', UserComponent> & {
		component: UserComponent;
		props?: Omit<ComponentProps<UserComponent>, 'item'>;
	};

	type StackItemPopVerboseInput = {
		id?: string;
		resolved?: any;
	};

	type StackItemRenderActionReturn = ActionReturn<StackItem<any>>;

	export { StackItem, Stack, stack, StackBuilder };
}

//# sourceMappingURL=index.d.ts.map